<?php
/**
 * @version     1.0.0
 * @package     com_docebo
 * @copyright   Copyright (C) 2013. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Docebo Srl. <info@docebo.com> - http://www.docebo.com
 */

// No direct access.
defined('_JEXEC') or die;

jimport('joomla.application.component.modelform');
jimport('joomla.event.dispatcher');

/**
 * Docebo model.
 */
class DoceboModelLoginForm extends JModelForm {

	var $_item = NULL;

	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @since    1.6
	 */
	protected function populateState() {
		$app = JFactory::getApplication('com_docebo');

		// Load state from the request userState on edit or from the passed variable on default
		if (JFactory::getApplication()->input->get('layout') == 'edit') {
			$id = JFactory::getApplication()->getUserState('com_docebo.edit.login.id');
		} else {
			$id = JFactory::getApplication()->input->get('id');
			JFactory::getApplication()->setUserState('com_docebo.edit.login.id', $id);
		}
		$this->setState('login.id', $id);

		// Load the parameters.
		$params       = $app->getParams();
		$params_array = $params->toArray();
		if (isset($params_array['item_id'])) {
			$this->setState('login.id', $params_array['item_id']);
		}
		$this->setState('params', $params);

	}


	/**
	 * Method to get an ojbect.
	 *
	 * @param    integer    The id of the object to get.
	 *
	 * @return    mixed    Object on success, false on failure.
	 */
	public function &getData($id = NULL) {
		if ($this->_item === NULL) {
			$this->_item = FALSE;

			if (empty($id)) {
				$id = $this->getState('login.id');
			}

			// Get a level row instance.
			$table = $this->getTable();

			// Attempt to load the row.
			if ($table->load($id)) {
				// Check published state.
				if ($published = $this->getState('filter.published')) {
					if ($table->state != $published) {
						return $this->_item;
					}
				}

				// Convert the JTable to a clean JObject.
				$properties  = $table->getProperties(1);
				$this->_item = JArrayHelper::toObject($properties, 'JObject');
			} elseif ($error = $table->getError()) {
				$this->setError($error);
			}
		}

		return $this->_item;
	}

	public function getTable($type = 'Login', $prefix = 'DoceboTable', $config = array()) {
		$this->addTablePath(JPATH_COMPONENT_ADMINISTRATOR . '/tables');

		return JTable::getInstance($type, $prefix, $config);
	}


	/**
	 * Method to check in an item.
	 *
	 * @param    integer        The id of the row to check out.
	 *
	 * @return    boolean        True on success, false on failure.
	 * @since    1.6
	 */
	public function checkin($id = NULL) {
		// Get the id.
		$id = (!empty($id)) ? $id : (int) $this->getState('login.id');

		if ($id) {

			// Initialise the table
			$table = $this->getTable();

			// Attempt to check the row in.
			if (method_exists($table, 'checkin')) {
				if (!$table->checkin($id)) {
					$this->setError($table->getError());

					return FALSE;
				}
			}
		}

		return TRUE;
	}

	/**
	 * Method to check out an item for editing.
	 *
	 * @param    integer        The id of the row to check out.
	 *
	 * @return    boolean        True on success, false on failure.
	 * @since    1.6
	 */
	public function checkout($id = NULL) {
		// Get the user id.
		$id = (!empty($id)) ? $id : (int) $this->getState('login.id');

		if ($id) {

			// Initialise the table
			$table = $this->getTable();

			// Get the current user object.
			$user = JFactory::getUser();

			// Attempt to check the row out.
			if (method_exists($table, 'checkout')) {
				if (!$table->checkout($user->get('id'), $id)) {
					$this->setError($table->getError());

					return FALSE;
				}
			}
		}

		return TRUE;
	}

	/**
	 * Method to get the profile form.
	 *
	 * The base form is loaded from XML
	 *
	 * @param    array   $data An optional array of data for the form to interogate.
	 * @param    boolean $loadData True if the form is to load its own data (default case), false if not.
	 *
	 * @return    JForm    A JForm object on success, false on failure
	 * @since    1.6
	 */
	public function getForm($data = array(), $loadData = TRUE) {
		// Get the form.
		$form = $this->loadForm('com_docebo.login', 'loginform', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return FALSE;
		}

		return $form;
	}

	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return    mixed    The data for the form.
	 * @since    1.6
	 */
	protected function loadFormData() {
		$data = $this->getData();

		return $data;
	}

	/**
	 * Method to save the form data.
	 *
	 * @param    array        The form data.
	 *
	 * @return    mixed        The user id on success, false on failure.
	 * @since    1.6
	 */
	public function save($data) {
		$id    = (!empty($data['id'])) ? $data['id'] : (int) $this->getState('login.id');
		$state = (!empty($data['state'])) ? 1 : 0;
		$user  = JFactory::getUser();

		if ($id) {
			//Check the user can edit this item
			$authorised = $user->authorise('core.edit', 'com_docebo') || $authorised = $user->authorise('core.edit.own', 'com_docebo');
			if ($user->authorise('core.edit.state', 'com_docebo') !== TRUE && $state == 1) { //The user cannot edit the state of the item.
				$data['state'] = 0;
			}
		} else {
			//Check the user can create new items in this section
			$authorised = $user->authorise('core.create', 'com_docebo');
			if ($user->authorise('core.edit.state', 'com_docebo') !== TRUE && $state == 1) { //The user cannot edit the state of the item.
				$data['state'] = 0;
			}
		}

		if ($authorised !== TRUE) {
			JError::raiseError(403, JText::_('JERROR_ALERTNOAUTHOR'));

			return FALSE;
		}

		$table = $this->getTable();
		if ($table->save($data) === TRUE) {
			return $id;
		} else {
			return FALSE;
		}

	}

	function delete($data) {
		$id = (!empty($data['id'])) ? $data['id'] : (int) $this->getState('login.id');
		if (JFactory::getUser()->authorise('core.delete', 'com_docebo') !== TRUE) {
			JError::raiseError(403, JText::_('JERROR_ALERTNOAUTHOR'));

			return FALSE;
		}
		$table = $this->getTable();
		if ($table->delete($data['id']) === TRUE) {
			return $id;
		} else {
			return FALSE;
		}

		return TRUE;
	}

}